"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var immer_1 = require("immer");
var PureStore = /** @class */ (function () {
    function PureStore(rootState, getter, root) {
        var _this = this;
        this.callbacks = [];
        this.getter = null;
        this.update = function (updater) {
            var updaterFn = updater;
            if (!(updater instanceof Function))
                updaterFn = function (e) { return Object.assign(e, updater); };
            _this.root.state = immer_1.default(_this.root.state, function (s) {
                updaterFn(_this.getter(s));
            });
            _this.state = _this.getter(_this.root.state);
            _this.callbacks.forEach(function (callback) { return callback(); });
        };
        this.storeFor = function (getter) {
            return new PureStore(_this.root.state, getter, _this);
        };
        this.updaterFor = function (getter) {
            return _this.storeFor(getter).update;
        };
        this.subscribe = function (callback) {
            if (_this.root != _this)
                throw "Only the top level store can be subscribed to.";
            _this.callbacks.push(callback);
            return function () { return _this.callbacks.splice(_this.callbacks.indexOf(callback), 1); };
        };
        this.root = root || this;
        this.getter = getter || (function (s) { return s; });
        this.state = this.getter(rootState);
    }
    return PureStore;
}());
exports.PureStore = PureStore;
exports.default = (function (state) { return (new PureStore(state, function (s) { return s; })); });
